const classes = {
  disabled: "disabled",
  inputError: "input__control--error",
  navActive: "nav--expanded",
};

const state = {};
const page = fetchElement("[data-page]").dataset.page;

switch (page) {
  case "login": {
    attachEvent(fetchById("js-login-form"), "submit", logUserIn);
    break;
  }
  case "update-password": {
    attachEvent(fetchById("js-user-form"), "submit", updatePassword);
    break;
  }
  case "add-blog": {
    const toolbarOptions = [
      ["bold", "italic", "underline", "strike", "link", "image"],
      [
        {
          header: [2, 3, 4, 5, 6, false],
        },
      ],
      [{ list: "ordered" }, { list: "bullet" }],
      ["clean"],
      ["blockquote"],
    ];

    var options = {
      placeholder: "Blog content goes here...",
      theme: "snow",
      modules: {
        toolbar: toolbarOptions,
      },
    };
    const editor = new Quill(".quill", options);
    state["editor"] = editor;

    state["editor"].getModule("toolbar").addHandler("image", () => {
      selectLocalImage();
    });

    document.getElementById("js-image").addEventListener("change", () => {
      const [file] = document.getElementById("js-image").files;
      if (file) {
        document.getElementById("js-preview").src = URL.createObjectURL(file);
      }
    });

    attachEvent(fetchById("js-create-blog-form"), "submit", uploadBlog);
    break;
  }
  case "modify-blog": {
    const toolbarOptions = [
      ["bold", "italic", "underline", "strike", "link", "image"],
      [
        {
          header: [2, 3, 4, 5, 6, false],
        },
      ],
      [{ list: "ordered" }, { list: "bullet" }],
      ["clean"],
      ["blockquote"],
    ];

    var options = {
      placeholder: "Blog content goes here...",
      theme: "snow",
      modules: {
        toolbar: toolbarOptions,
      },
    };
    const editor = new Quill(".quill", options);
    state["editor"] = editor;

    state["editor"].getModule("toolbar").addHandler("image", () => {
      selectLocalImage();
    });

    document.getElementById("js-image").addEventListener("change", () => {
      const [file] = document.getElementById("js-image").files;
      if (file) {
        document.getElementById("js-preview").src = URL.createObjectURL(file);
      }
    });

    attachEvent(fetchById("js-update-btn"), "click", updateBlog);
    attachEvent(fetchById("js-delete-btn"), "click", deleteBlog);
    break;
  }
}

// CREATE BLOG STUFF
// Create Image Instance for QuillJS
function selectLocalImage() {
  const input = document.createElement("input");
  input.setAttribute("type", "file");
  input.setAttribute("accept", ".png, .jpg, .jpeg");
  input.click();
  // LISTEN UPLOAD LOCAL IMAGE AND SAVE TO SERVER
  input.onchange = () => {
    const file = input;
    saveToServer(file);
  };
}

// Send file to server
function saveToServer(file) {
  const fd = new FormData();
  fd.append("operation", "uploadFile");
  fd.append("image", file.files[0]);

  const xhr = new XMLHttpRequest();
  xhr.open("POST", "php/apis/blog", true);
  xhr.onload = () => {
    if (xhr.status === 200) {
      const response = xhr.responseText;
      console.log(response);
      const url = JSON.parse(response).url;
      insertToEditor(url);
    }
  };
  xhr.send(fd);
}

// Push received URL to editor
function insertToEditor(url) {
  // PUSH IMAGE URL TO RICH EDITOR.
  console.log(url);
  let range = state["editor"].getSelection();
  state["editor"].insertEmbed(range.index, "image", url);
  state["editor"].setSelection(range.index + 1);

  range = state["editor"].getSelection();
  state["editor"].insertText(range.index, "\n");
}

// LOGIN
function logUserIn(e) {
  e.preventDefault();

  const email = document.getElementById("js-email");
  const password = document.getElementById("js-password");

  if (verifyInput(email, "email") & verifyInput(password)) {
    const payload = generatePayload({
      operation: "login-admin",
      email: email.value,
      password: password.value,
    });

    disableButtons(".btn");
    pushToServer(payload, "user").then((response) => {
      if (response.success) {
        redirect("index");
      } else {
        Toast.show(response.message, "error");
      }
      enableButtons(".btn");
    });
  } else {
    Toast.show("Please fill in all mandatory fields", "error");
  }
}

// PASSWORD
function updatePassword(e) {
  e.preventDefault();

  const password = document.getElementById("js-password");
  const confirmPassword = document.getElementById("js-confirm-password");

  if (verifyInput(password) & verifyPassword(password, confirmPassword)) {
    const payload = generatePayload({
      operation: "update-password",
      password: password.value,
    });

    disableButtons(".btn");
    pushToServer(payload, "user").then((response) => {
      if (response.success) {
        Toast.show(response.message, "success");
        enableButtons(".btn");
      } else {
        Toast.show(response.message, "error");
      }
    });
  } else {
    Toast.show("Passwords do not match or are empty...", "error");
  }
}

// BLOGS
function uploadBlog(e) {
  e.preventDefault();
  const title = document.getElementById("js-title");
  let titleErr = false;

  const thumbnail = document.getElementById("js-image");
  let thumbnailErr = false;

  const quill =
    state["editor"].root.innerHTML == "<p><br></p>"
      ? ""
      : state["editor"].root.innerHTML;
  let quillErr = false;

  const metaDescription = document.getElementById("js-description");

  if (title.value.trim().length == 0) {
    titleErr = true;
    attachClass(title, "input__control--error");
  } else {
    detachClass(title, "input__control--error");
  }

  if (thumbnail.value.trim().length == 0) {
    thumbnailErr = true;
    attachClass(thumbnail, "input__control--error");
  } else {
    detachClass(thumbnail, "input__control--error");
  }

  if (!quill) {
    quillErr = true;
    document
      .getElementById("js-blog-content")
      .classList.add("input__control--error");
  } else {
    document
      .getElementById("js-blog-content")
      .classList.remove("input__control--error");
  }

  if (!titleErr && !thumbnailErr && !quillErr) {
    disableButtons(".btn");

    const payload = generatePayload({
      operation: "add-blog",
      title: title.value,
      thumbnail: thumbnail.files[0],
      content: quill,
      metaDescription: metaDescription.value,
    });
    if (payload) {
      pushToServer(payload, "blog").then((response) => {
        if (response.success) {
          Toast.show(response.message, "success");
        } else {
          Toast.show(response.message, "error");
                  enableButtons(".btn");

        }

      });
    }
  } else {
    Toast.show("Please fill in all the mandatory fields", "error");
  }
}

function updateBlog() {
  const id = this.parentNode.dataset.id;
  const title = fetchById("js-title");
  let titleErr = false;

  let thumbnail = fetchById("js-image");

  const quill =
    state["editor"].root.innerHTML == "<p><br></p>"
      ? ""
      : state["editor"].root.innerHTML;
  let quillErr = false;

  const metaDescription = fetchById("js-description");

  if (title.value.trim().length == 0) {
    titleErr = true;
    attachClass(title, "input__control--error");
  } else {
    detachClass(title, "input__control--error");
  }

  if (thumbnail.value.trim().length == 0) {
    thumbnail = "";
  } else {
    thumbnail = thumbnail.files[0];
  }

  if (!quill) {
    quillErr = true;
    document
      .getElementById("js-blog-content")
      .classList.add("input__control--error");
  } else {
    document
      .getElementById("js-blog-content")
      .classList.remove("input__control--error");
  }

  if (!titleErr && !quillErr) {
    const payload = generatePayload({
      operation: "update-blog",
      id: id,
      title: title.value,
      thumbnail: thumbnail,
      content: quill,
      metaDescription: metaDescription.value,
    });
    if (payload) {
      disableButtons(".btn");
      pushToServer(payload, "blog", {log:true}).then((response) => {
        enableButtons(".btn");
        if (response.success) {
          Toast.show(response.message, "success");
        } else {
          Toast.show(response.message, "error");
        }
      });
    }
  } else {
    Toast.show("Please fill in all the mandatory fields", "error");
  }
}

function deleteBlog() {
  const id = this.parentNode.dataset.id;
  const payload = generatePayload({
    operation: "delete-blog",
    id: id,
  });

  if (payload) {
    disableButtons(".btn");
    pushToServer(payload, "blog").then((response) => {
      if (response.success) {
        Toast.show(response.message, "success");
        setTimeout(() => {
          window.location.href = "select-blog";
        }, 2000);
      } else {
        enableButtons(".btn");
        Toast.show(response.message, "error");
      }
    });
  }
}

// 1 | HELPER FUNCTIONS
/*
 * 1.1 | Create FormData from object [generatePayload]
 * Desc: Pass in object. Returns FormData with key:value pair
 */
function generatePayload(obj) {
  const formData = new FormData();
  for (let key in obj) {
    formData.append(key, obj[key]);
  }
  return formData;
}

/*
 * 1.2 | Push data to server and redirect to callback function [pushToServer]
 * Desc: Push to PHP server and pass response to callback function
 */
async function pushToServer(payload, apiName, options = {}) {
  if (!options.method) {
    options.method = "POST";
  }

  var url = `php/apis/${apiName}.php`;

  const response = await fetch(url, {
    method: options.method,
    body: payload,
  });

  if (options.log) {
    console.log(await response.text());
    return;
  }

  return await response.json();
}

/*
 * 1.3 | Redirect [redirect]
 * Desc: Redirect to url
 */
function redirect(url) {
  window.location.href = url;
  return;
}

/*
 * 1.4 | Print entire Form Data [printFormData]
 * Desc: Redirect to url
 */
function printFormData(payload) {
  for (var pair of payload.entries()) {
    console.log(pair[0] + ", " + pair[1]);
  }
}

////////////////////////////////////////////////////////////////////////////
// 2 | BUTTON INTERACTIONS

/*
 * 2.1 | Disable and add loading animation to button [disableBtn]
 * Desc: Disable Buttons by class name
 */
function disableButtons(btnClass, options = {}) {
  const buttons = fetchElements(btnClass);

  if (options.el) {
    attachClass(options.el, classes.targetBtnClass);

    if (options.text) {
      options.el.innerText = options.text;
    }
  }

  for (let i = 0; i < buttons.length; i++) {
    buttons[i].disabled = true;
  }
}

/*
 * 2.2 | Enable Button
 * Desc: Companion call for disableButtons()
 */
function enableButtons(btnClass, options = {}) {
  const buttons = fetchElements(btnClass);

  const el = fetchElement(`${btnClass} + .${classes.targetBtnClass}`);

  for (let i = 0; i < buttons.length; i++) {
    buttons[i].disabled = false;
  }

  if (el) {
    detachClass(el, classes.targetBtnClass);

    if (options.text) {
      el.innerText = options.text;
    }

    if (options.keepDisabled) {
      el.disabled = true;
    }
  }
}

////////////////////////////////////////////////////////////////////////////
// 3 | SELECTOR OPERATIONS

/*
 * 3.1 | Fetch Element by ID [fetchById]
 */
function fetchById(elString) {
  return document.getElementById(elString);
}

/*
 * 3.2 | Fetch Element by Selector / Class
 */
function fetchElement(elString, rootEl = document) {
  return rootEl.querySelector(elString);
}

/*
 * 3.3 | Fetch (many) Elements by Selector / Class
 */
function fetchElements(elString, src = document) {
  return src.querySelectorAll(elString);
}

////////////////////////////////////////////////////////////////////////////
// 4 | CLASS INTERACTIONS

/*
 * 4.1 | Add class to Element
 */
function attachClass(el, className) {
  el.classList.add(className);
}

/*
 * 4.2 | Remove class from Element
 */
function detachClass(el, className) {
  el.classList.remove(className);
}

/*
 * 4.3 | Check if element contains class
 */
function containsClass(el, className) {
  return el.classList.contains(className);
}

/**************************************************************************/

/*
 * 5.3 | Animate snackbar
 */
function displayErrSnackbar() {
  const snackbarEl = fetchById("snackbar");
  attachClass(snackbarEl, classes["snackbar-visible"]);

  snackbarEl.addEventListener("animationend", function () {
    detachClass(this, classes["snackbar-visible"]);
  });
}

////////////////////////////////////////////////////////////////////////////
// 6 | EVENT LISTENERS

/*
 * 6.1 | Attach event to single element
 */
function attachEvent(el, event, callback) {
  el.addEventListener(event, callback);
}

/*
 * 6.2 | Attach event to multiple elements
 */
function attachEventMultiple(elArr, event, callback) {
  for (let i = 0; i < elArr.length; i++) {
    attachEvent(elArr[i], event, callback);
  }
}

/**
 * 6.3 | Detach Event / Remove Event
 */
function detachEvent(el, event, callback) {
  el.removeEventListener(event, callback);
}

function addErrorClassToInput(el) {
  attachClass(el, classes.inputError);
}

function removeErrorClassFromInput(el) {
  detachClass(el, classes.inputError);
}

function validateEmail(email) {
  const re =
    /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
  return re.test(String(email).toLowerCase());
}

function verifyInput(el, type = "text") {
  if (type == "email") {
    if (validateEmail(el.value)) {
      removeErrorClassFromInput(el);
      return true;
    } else {
      addErrorClassToInput(el);
      return false;
    }
  } else if (type == "text") {
    if (el.value.trim() != "") {
      removeErrorClassFromInput(el);
      return true;
    } else {
      addErrorClassToInput(el);
      return false;
    }
  } else if (type == "file") {
    if (el.value.trim() != "") {
      removeErrorClassFromInput(el);
      return true;
    } else {
      addErrorClassToInput(el);
      return false;
    }
  }
}

function verifyPassword(password1, password2) {
  if (password1.value.trim() == "") {
    addErrorClassToInput(password1);
    return false;
  } else {
    removeErrorClassFromInput(password1);
  }

  if (password1.value == password2.value) {
    removeErrorClassFromInput(password2);
    return true;
  } else {
    addErrorClassToInput(password2);
    return false;
  }
}

function printFormData(payload) {
  for (var pair of payload.entries()) {
    console.log(pair[0] + ", " + pair[1]);
  }
}
