<?php
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);


include_once '../functions.php';
include_once '../uploader.php';

if ($_POST["operation"] == "add-blog") {
    $title = htmlspecialchars($_POST["title"]);
    $short_url = generateShortString($title);
    $content = $_POST["content"];
    $metaDescription = htmlspecialchars($_POST["metaDescription"]);

    if (isset($_FILES['thumbnail'])) {
        $featuredImg = uploadImage('thumbnail', true);
    } else {
        $response->success = false;
        $response->message = "Something went wrong with the upload process";
        exit(json_encode($response));
    }

    $query =
        "INSERT INTO blog(title, short_title, content, meta_description, image, thumbnail) VALUES (:title, :short_url, :content, :metaDescription, :featuredImg, :thumbnail)";

    $stmt = $pdo->prepare($query);
    $stmt->bindValue(":title", $title, PDO::PARAM_STR);
    $stmt->bindValue(":short_url", $short_url, PDO::PARAM_STR);
    $stmt->bindValue(":content", $content, PDO::PARAM_STR);
    $stmt->bindValue(":metaDescription", $metaDescription, PDO::PARAM_STR);
    $stmt->bindValue(":featuredImg", BASEURL . $featuredImg->fullPath, PDO::PARAM_STR);
    $stmt->bindValue(":thumbnail", BASEURL . $featuredImg->thumbURL, PDO::PARAM_STR);

    $stmt->execute();

    if ($stmt) {
        $response->success = true;
        $response->message = "The blog was added into the database!";
    } else {
        $response->success = false;
        $response->message = "Something went wrong...";
    }

    exit(json_encode($response));
} elseif ($_POST["operation"] == "update-blog") {
    $id = filter_var($_POST['id'], FILTER_SANITIZE_NUMBER_INT);
    $title = htmlspecialchars($_POST["title"]);
    $short_url = generateShortString($title);
    $content = $_POST["content"];
    $metaDescription = htmlspecialchars($_POST["metaDescription"]);

    if (isset($_FILES['thumbnail'])) {
        $featuredImg = uploadImage('thumbnail', true);

        $query =
            "UPDATE blog SET title = :title, short_title = :short_url, content = :content, meta_description = :metaDescription, image = :featuredImg, thumbnail = :thumbnail WHERE id = :id";

        $stmt = $pdo->prepare($query);
        $stmt->bindValue(":title", $title, PDO::PARAM_STR);
        $stmt->bindValue(":short_url", $short_url, PDO::PARAM_STR);
        $stmt->bindValue(":content", $content, PDO::PARAM_STR);
        $stmt->bindValue(":metaDescription", $metaDescription, PDO::PARAM_STR);
        $stmt->bindValue(":featuredImg", BASEURL . $featuredImg->fullPath, PDO::PARAM_STR);
        $stmt->bindValue(":thumbnail", BASEURL . $featuredImg->thumbURL, PDO::PARAM_STR);
        $stmt->bindValue(":id", $id, PDO::PARAM_INT);

        $stmt->execute();
    } else {
        $query =
            "UPDATE blog SET title = :title, short_title = :short_url, content = :content, meta_description = :metaDescription WHERE id = :id";

        $stmt = $pdo->prepare($query);
        $stmt->bindValue(":title", $title, PDO::PARAM_STR);
        $stmt->bindValue(":short_url", $short_url, PDO::PARAM_STR);
        $stmt->bindValue(":content", $content, PDO::PARAM_STR);
        $stmt->bindValue(":metaDescription", $metaDescription, PDO::PARAM_STR);
        $stmt->bindValue(":id", $id, PDO::PARAM_INT);

        $stmt->execute();
    }

    if ($stmt) {
        $response->success = true;
        $response->message = "The blog was updated!";
    } else {
        $response->success = false;
        $response->message = "Something went wrong...";
    }

    exit(json_encode($response));
} elseif ($_POST["operation"] == "delete-blog") {
    $blogId = filter_var($_POST["id"], FILTER_SANITIZE_NUMBER_INT);

    $query = "DELETE FROM blog WHERE id = :blogId";
    $stmt = $pdo->prepare($query);
    $stmt->bindValue(":blogId", $blogId, PDO::PARAM_INT);
    $stmt->execute();

    if ($stmt) {
        $response->success = true;
        $response->message = "The blog has been removed from the database! Please wait while we redirect you.";
    } else {
        $response->success = false;
        $response->message = "Something went wrong...";
    }

    exit(json_encode($response));
} elseif ($_POST['operation'] == 'uploadFile') {
    if (isset($_FILES['image'])) {
        $image = uploadImage('image');
        $response->url = BASEURL . $image->fullPath;

        exit(json_encode($response));
    }
}
