<?php
////////////////////////////////////////////
// FILE UPLOADS
function byteconvert($input)
{
    preg_match('/(\d+)(\w+)/', $input, $matches);
    $type = strtolower($matches[2]);
    switch ($type) {
    case "b":
        $output = $matches[1];
        break;
    case "kb":
        $output = $matches[1]*1024;
        break;
    case "mb":
        $output = $matches[1]*1024*1024;
        break;
    case "gb":
        $output = $matches[1]*1024*1024*1024;
        break;
    case "tb":
        $output = $matches[1]*1024*1024*1024;
        break;
    }
    return $output;
}


function getImageFromMime($mime, $source_url)
{
    if ($mime == 'image/jpeg') {
        $image_p = imagecreatefromjpeg($source_url);
    } elseif ($mime == 'image/gif') {
        $image_p = imagecreatefromgif($source_url);
    } elseif ($mime == 'image/png') {
        $image_p = imagecreatefrompng($source_url);
    }
    return $image_p;
}


function compressImage($source_url, $destination_url, $quality) 
{
      $filesize = filesize($source_url);
      if ($filesize > byteconvert('1mb')) {
            $percent = null;
            $info = getimagesize($source_url);

            // Find image dimensions
            list($width, $height) = getimagesize($source_url);
            $result_dimension = IMGWIDTH;

            // Resize creteria
            if ($width >= IMGWIDTH) {
                  $percent = $result_dimension / $width;
            } elseif ($height >= IMGHEIGHT) {
                  $percent = $result_dimension / $height;
            }
            if (!empty($percent)) {
                  $new_width = round($width * $percent);
                  $new_height = round($height * $percent);
                  
                  // Resample
                  $image_p = imagecreatetruecolor($new_width, $new_height);
                  $image = getImageFromMime($info['mime'], $source_url);
                  imagecopyresampled($image_p, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
            } else {
                  $image_p = getImageFromMime($info['mime'], $source_url);
            }
            imagejpeg($image_p, $destination_url, $quality);
            return $destination_url;
      } else {
            move_uploaded_file($source_url, $destination_url);
            return $destination_url;
      }
}


function generateThumbnail($source_url, $destination_url, $quality) 
{
      $percent = null;
      $info = getimagesize($source_url);

      // Find image dimensions
      list($width, $height) = getimagesize($source_url);
      $result_dimension = THUMBWIDTH;

      // Resize creteria
      if ($width >= THUMBWIDTH) {
            $percent = $result_dimension / $width;
      } elseif ($height >= THUMBHEIGHT) {
            $percent = $result_dimension / $height;
      }

      if (!empty($percent)) {
            $new_width = round($width * $percent);
            $new_height = round($height * $percent);
            // Resample
            $image_p = imagecreatetruecolor($new_width, $new_height);
            $image = getImageFromMime($info['mime'], $source_url);
            imagecopyresampled($image_p, $image, 0, 0, 0, 0, $new_width, $new_height, $width, $height);
      } else {
            $image_p = getImageFromMime($info['mime'], $source_url);
      }
      imagejpeg($image_p, $destination_url, $quality);
      return $destination_url;
}

function createThumbnail($fileString) 
{
      $response = new stdClass;

      $fileExtensions = ['jpeg', 'jpg', 'png', 'JPG', 'PNG']; 
  
      $fileName = $_FILES[$fileString]['name'];
      $fileSize = $_FILES[$fileString]['size'];
      $fileTmpName  = $_FILES[$fileString]['tmp_name'];
      $fileType = $_FILES[$fileString]['type'];
      $temp = explode('.', $fileName);
      $fileExtension = end($temp);
  
      $fullpath = BASEURL . UPLOADFOLDER . '/';
      
      // Create folders by the date (ex: June, 2021)
      $date = date('Y-m-d');
      $relPath = setupFolder($date, UPLOADPATH);

      // Check if file type exists in the whitelisted array
      if (!in_array($fileExtension, $fileExtensions)) {
            $response->success = false;
            $response->message = "This file extension is not allowed. Please upload a JPEG or PNG file";
            return $response;
      }
      
      $uploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);
      $image = generateThumbnail($_FILES[$fileString]["tmp_name"], UPLOADPATH . $uploadPath, 80);

      if($image) {
            $response->success = true;
            $response->fullPath = $fullpath . $uploadPath;
      } else {
            $response->success = false;
            $response->message = 'Something went wrong with the upload process.';
      }

      return $response;
}

function uploadImage($fileString, $isThumbnailRequired = false) 
{
      $response = new stdClass;

      $fileExtensions = ['jpeg', 'jpg', 'png', 'JPG', 'PNG']; 
  
      $fileName = $_FILES[$fileString]['name'];
      $fileSize = $_FILES[$fileString]['size'];
      $fileTmpName  = $_FILES[$fileString]['tmp_name'];
      $fileType = $_FILES[$fileString]['type'];
      $temp = explode('.', $fileName);
      $fileExtension = end($temp);
  
      $fullpath = UPLOADFOLDER . '/';
      
      // Create folders by the date (ex: June, 2021)
      $date = date('Y-m-d');
      $relPath = setupFolder($date, UPLOADPATH);

      // Check if file type exists in the whitelisted array
      if (!in_array($fileExtension, $fileExtensions)) {
            $response->success = false;
            $response->message = "This file extension is not allowed. Please upload a JPEG or PNG file";
            return $response;
      }
  
      // Compress Image     
      if($isThumbnailRequired) {
            $thumbUploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);
            $thumbnail = generateThumbnail($_FILES[$fileString]["tmp_name"], UPLOADPATH . $thumbUploadPath, 80);
      }
      
      $uploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);
      $image = compressImage($_FILES[$fileString]["tmp_name"], UPLOADPATH . $uploadPath, 80);

      if($image) {
            $response->success = true;
            $response->fullPath = $fullpath . $uploadPath;

            if($isThumbnailRequired) {
                  $response->thumbURL = $fullpath . $thumbUploadPath;
            }
      } else {
            $response->success = false;
            $response->message = 'Something went wrong with the upload process.';
      }

      return $response;
}

function uploadImageMultiple($fileString, $i, $isThumbnailRequired = false) 
{
      $response = new stdClass;

      $fileExtensions = ['jpeg', 'jpg', 'png', 'JPG', 'PNG']; 
  
      $fileName = $_FILES[$fileString]['name'][$i];
      $fileSize = $_FILES[$fileString]['size'][$i];
      $fileTmpName  = $_FILES[$fileString]['tmp_name'][$i];
      $fileType = $_FILES[$fileString]['type'][$i];
      $temp = explode('.', $fileName);
      $fileExtension = end($temp);
  
      $fullpath = UPLOADFOLDER . '/';
      
      // Create folders by the date (ex: June, 2021)
      $date = date('Y-m-d');
      $relPath = setupFolder($date, UPLOADPATH);

      // Check if file type exists in the whitelisted array
      if (!in_array($fileExtension, $fileExtensions)) {
            $response->success = false;
            $response->message = "This file extension is not allowed. Please upload a JPEG or PNG file";
            return $response;
      }
  
      // Compress Image     
      if($isThumbnailRequired) {
            $thumbUploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);
            $thumbnail = generateThumbnail($_FILES[$fileString]["tmp_name"][$i], UPLOADPATH . $thumbUploadPath, 80);
      }
      
      $uploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);
      $image = compressImage($_FILES[$fileString]["tmp_name"][$i], UPLOADPATH . $uploadPath, 80);

      if($image) {
            $response->success = true;
            $response->fullPath = $fullpath . $uploadPath;

            if($isThumbnailRequired) {
                  $response->thumbURL = $fullpath . $thumbUploadPath;
            }
      } else {
            $response->success = false;
            $response->message = 'Something went wrong with the upload process.';
      }

      return $response;
}

function uploadWithoutCompression($fileString) 
{
    $response = new stdClass;

    $fileExtensions = ['jpeg', 'jpg', 'png', 'JPG', 'PNG', 'pdf', 'PDF', 'mp3', 'MP3']; 

    $fileName = $_FILES[$fileString]['name'];
    $fileSize = $_FILES[$fileString]['size'];
    $fileTmpName  = $_FILES[$fileString]['tmp_name'];
    $fileType = $_FILES[$fileString]['type'];
    $temp = explode('.', $fileName);
    $fileExtension = end($temp);

    $fullpath = BASEURL . UPLOADFOLDER . '/';
    $date = date('Y-m-d');
    $relPath = setupFolder($date, UPLOADPATH);

    if (!in_array($fileExtension, $fileExtensions)) {
          $response->success = false;
          $response->message = "This file extension is not allowed. Please upload a JPEG, PNG, or PDF file";
          return $response;
    }
    
    $uploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);

    if(move_uploaded_file($_FILES[$fileString]["tmp_name"], UPLOADPATH . $uploadPath)) {
          $response->success = true;
          $response->fullPath = $fullpath . $uploadPath;
    } else {
          $response->success = false;
          $response->message = 'Something went wrong with the upload process.';
    }

    return $response;
}

function uploadPdf($fileString) 
{
      $response = new stdClass;

      $fileExtensions = ['pdf']; 
  
      $fileName = $_FILES[$fileString]['name'];
      $fileSize = $_FILES[$fileString]['size'];
      $fileTmpName  = $_FILES[$fileString]['tmp_name'];
      $fileType = $_FILES[$fileString]['type'];
      $temp = explode('.', $fileName);
      $fileExtension = end($temp);
  
      $fullpath = UPLOADFOLDER . '/';
      
      // Create folders by the date (ex: June, 2021)
      $date = date('Y-m-d');
      $relPath = setupFolder($date, UPLOADPATH);

      // Check if file type exists in the whitelisted array
      if (!in_array($fileExtension, $fileExtensions)) {
            $response->success = false;
            $response->message = "This file extension is not allowed. Please upload a PDF";
            return $response;
      }
  
      $uploadPath = $relPath . basename(generateRandomString() . '.' . $fileExtension);

      

      if(move_uploaded_file($_FILES[$fileString]["tmp_name"], UPLOADPATH . $uploadPath)) {
            $response->success = true;
            $response->fullPath = $fullpath . $uploadPath;
      } else {
            $response->success = false;
            $response->message = 'Something went wrong with the upload process.';
      }

      return $response;
}

////////////////////////////////////////////
// FOLDER LOGIC

function setupFolder($date)
{
    // $mod = 0755;
    $baseFolder = UPLOADPATH;
    $time = strtotime($date);
    $month = date("F", $time);
    $year = date("Y", $time);
    // Check if folder exists
    $pwd = checkFolder($year, $baseFolder);
    if ($pwd) {
          $pwd = checkSubFolder($pwd . $month, $baseFolder);
          if ($pwd) {
                return $pwd;
          } else {
                return false;
          }
    } else {
          return false;
    }
    return false;
}


function checkFolder($year, $baseFolder, $mod = 0755)
{
    $pwd = '';
    if (!file_exists($baseFolder . $year)) {
    //echo 'Folder does not exist' . PHP_EOL;
        if (mkdir($baseFolder . $year, $mod)) {
    // Create new folder (using the year) and set it as the pwd
            $pwd = $year . '/';
        }
    } else {
        //echo 'Folder exists' . PHP_EOL;
        $pwd = $year . '/';
    }
    return (!empty($pwd)) ? $pwd : false;
}


function checkSubFolder($month, $baseFolder, $mod = 0755)
{
    $pwd = '';
    // Check if Sub-folder Exists
    if (!file_exists($baseFolder . $month)) {
    //echo 'Sub-folder does not exist' . PHP_EOL;
        if (mkdir($baseFolder . $month, $mod)) {
    // Create new folder (using the year) and set it as the pwd
            $pwd = $month . '/';
        }
    } else {
    //echo 'Folder exists' . PHP_EOL;
        $pwd = $month . '/';
    }
    return (!empty($pwd)) ? $pwd : false;
}
?>